import * as React from 'react';
import { IEditorRegistry, CacheTypenames } from '@workbench-stack/core';
import { TextResourceEditor, UntitledResourceEditor } from '@files-stack/client-react';
import { ClientExtensions } from '@workbench-stack/module-browser';
import { IEditorInputFactoryRegistry, IEditorInputFactory } from '@workbench-stack/core';
import { Registry } from '@vscode/monaco-editor/esm/vs/platform/registry/common/platform';
import {
    IFileEditorInput, FileClientTypenames, EditorsFileRefFragmentDoc, IEditorsFileRefFragment,
    IEDITOR_INPUT_ID, IIEditorInput, IEditorInput,
} from '@files-stack/core';
import { InMemoryCache } from 'apollo-cache-inmemory';
import { interfaces } from 'inversify';
import { ClientTypes } from '@common-stack/client-core';
import { FileEditorInputFactory } from './file-editor-input-registry';

export const FilesContribution = props => {

    Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
        id: TextResourceEditor.defaultProps.ID,
        name: TextResourceEditor.displayName,
    }, [{ __typename: FileClientTypenames.FileEditorInput as any }]);

    Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
        id: UntitledResourceEditor.defaultProps.ID,
        name: UntitledResourceEditor.displayName,
    }, [{ __typename: CacheTypenames.UntitledEditorInput }]);

    Registry.as<IEditorInputFactoryRegistry>(ClientExtensions.EditorInputFactories)
    .registerEditorInputFactory(IEDITOR_INPUT_ID.file, new FileEditorInputFactory());

    Registry.as<IEditorInputFactoryRegistry>(ClientExtensions.EditorInputFactories).registerFileInputFactory({
        createFileInput: (resource, encoding, mode, container: interfaces.Container): IFileEditorInput => {
            const utility: any = container.get(ClientTypes.UtilityClass);
            const cache = container.get<InMemoryCache>(ClientTypes.InMemoryCache);
            const id = utility.getCacheKey({ resource, __typename: FileClientTypenames.FileStatWithMetadata });
            const file = cache.readFragment<IEditorsFileRefFragment>({
                id,
                fragment: EditorsFileRefFragmentDoc,
                variables: { skipValue: true },
            });
            console.log('Fix it: Here types are not matching. It need provide resource as string instead we have ', file);
            // return file as IFileEditorInput;
            return file as any;
        },
    
        isFileInput: (obj): obj is IFileEditorInput => {
            return obj.__typename === FileClientTypenames.FileEditorInput;
        },
    });


    return(
        <>
            <TextResourceEditor {...props} />
            <UntitledResourceEditor {...props} />
        </>
    );
};

