import * as React from 'react';
import { Feature } from '@common-stack/client-react';
import { Files,  /** Editors */ } from '@files-stack/client-react';
import { reducers, setRootPath, filesModule } from '@files-stack/client-state';
import {
    filesResolvers, defaults, editorResolvers, filesTreeResolvers, untitledEditorResolver, dataIdFromObject,
    modelContentChangeEpic, modelSaveEpic, fragmentMatcher,
    schema,
} from '@files-stack/client-state';
import { merge } from 'lodash';
import { IClientFileService, ClientTypes, IFilesTreeService, IUntitledEditorService,
     IClientService, IClientTextFileService, ITextFileEditorModel, IUntitledFileEditorModel,
     } from '@files-stack/core';
import { DockLayout } from '@ide-layout/client-react';
import { interfaces } from 'inversify';
import { IEditorInput, IConfigurationService } from '@workbench-stack/core';
import { Editor } from '@workbench-stack/module-browser';
import { useGetEditorStateMutation, useUpdateEditorStateMutation } from '@files-stack/client-react';

const actionFunctions = [setRootPath];

const filesServiceGen = (container: interfaces.Container): IClientService => {
    return {
        fileService: container.get<IClientFileService>(ClientTypes.IFileService),
        textFileService: container.get<IClientTextFileService>(ClientTypes.ITextFileService),
        filesTreeService: container.get<IFilesTreeService>(ClientTypes.IFilesTreeService),
        untitledEditorService: container.get<IUntitledEditorService>(ClientTypes.IUntitledEditorService),
        textFileEditorModel: container.get<ITextFileEditorModel>(ClientTypes.ITextFileEditorModel),
        untitledFileEditorModel: container.get<IUntitledFileEditorModel>(ClientTypes.IUntitledFileEditorModel),
    };
};

// const Editor = ({ input }) => React.useMemo(() => EditorComponent({ input }), [input]);

const Content = ({ input }: { input: IEditorInput }) => {
    console.log('--Content Component should only render once per input', input);
    let props = {} as any;
    return  <Editor {...props} input={input} uri={input.resource} enableApi={true} />;
  };

const DockLayoutComponent = () => {
    const [getEditorState] = useGetEditorStateMutation();
    const [updateEditorState] = useUpdateEditorStateMutation();
    return (
        <DockLayout
            contentComponent={Content}
            getEditorState={getEditorState as any}
            updateEditorState={updateEditorState as any}
        />
    );
};
  
export default new Feature({
    leftMainPanelItems: { files: <Files /> },
    middleMainPanelItems: { editors: <DockLayoutComponent /> }, // <Editor />
    middleMainPanelItemsProps: { editors: { actionFunctions } },
    reducer: { ...reducers },
    createContainerFunc: filesModule,
    clientStateParams: [{ resolvers: [filesResolvers, editorResolvers, filesTreeResolvers, untitledEditorResolver],
        defaults, typeDefs: schema, fragmentMatcher: fragmentMatcher as any }],
    dataIdFromObject,
    createServiceFunc: filesServiceGen,
    epic: [modelContentChangeEpic, modelSaveEpic],
});
